#!/bin/sh -x
#
# Tomato STM Monitor <br> mod_rs232 v2 - 27/03/2013
# based on tievolu v1.12
# http://www.tievolu.co.uk/stm/stm-monitor.html
# Latest modified by rs232 on 27/03/2013. All the credits to Tievolu for his fantastic script!
#
# changes since 1.12
# 1) removed prevention, now the script accepts no parameters other then the mandatory contract speed
# 2) solved a bug with sentence end hour missing
# 3) redesign + progress bar
# 4) minor optimisation
# 5) Use of tmp file (stm-monitor.var)
# 6) simplified the contract definition to speed up future changes if any 
# 7) added reference to hh:mm left to the end of the sentence
# 8) added reference to last STM end time
# 9) changes to the crontab definition (see below)
# 10) added reference to transferred data after the period if ended
# 11) Added reference to P2P/newsgroup restriction enforcement
# 12) 
#
# This script is for use on routers running Tomato, connected to the
# Internet through a Virgin Media UK cable connection. VM's cable
# connections implement Subscriber Traffic Management (STM) i.e. the
# WAN bandwidth is limited for a set time (typically five hours) when
# a predefined amount of data is transferred between certain hours of
# the day.
#
#
# Add the following to your "WAN Up" script to download the script and
# set it to run every two minutes:
#
#    wget -O /tmp/stm-monitor.sh http://www.tomatothemebase.eu/wp-uploads
#    chmod 755 /tmp/stm-monitor.sh
#    cru a STM-Monitor-day "1-59/5 10-21 * * * /tmp/stm-monitor.sh [broadband type]"
#    cru a STM-Monitor-evening "10,30,50 22-23 * * * /tmp/stm-monitor.sh [broadband type]"
#    cru a STM-Monitor-night "55 0-9 * * * /tmp/stm-monitor.sh [broadband type]"
#    logger -t STM-Monitor "Downloaded `head /tmp/stm-monitor.sh | grep \"STM Monitor\" | sed 's/# //g'`"
#
#    Where [broadband type] = "S5", "M10", "L10", "XL20", "ML20", "L30", "XL30", "XXL50", "XL60", "XXL100" , "100" or "120" (no quotes)         
#
#	If unsure visit the following links:
#   http://www.virginmedia.com/images/STM30MblargeA.jpg
# 	http://www.virginmedia.com/images/STM20MblargeA.jpg
#
# You can optionally also add the following code to your WAN up script, 
# which allows you to customise some of the settings:
#
#    echo "STM_PERIOD_BW_LIMIT_1=" >> /tmp/stm-monitor.cfg
#    echo "STM_PERIOD_BW_LIMIT_2=" >> /tmp/stm-monitor.cfg
#    echo "STM_PERIOD_BW_LIMIT_3=" >> /tmp/stm-monitor.cfg
#    echo "STM_INBOUND_BANDWIDTH_WITHOUT_STM=" >> /tmp/stm-monitor.cfg
#    echo "STM_INBOUND_BANDWIDTH_WITH_STM=" >> /tmp/stm-monitor.cfg
#    echo "STM_OUTBOUND_BANDWIDTH_WITHOUT_STM=" >> /tmp/stm-monitor.cfg
#    echo "STM_OUTBOUND_BANDWIDTH_WITH_STM=" >> /tmp/stm-monitor.cfg
#    echo "QOS_IN_PERCENTAGE=" >> /tmp/stm-monitor.cfg
#    echo "QOS_OUT_PERCENTAGE=" >> /tmp/stm-monitor.cfg
#
#
# Events are recorded in the system log, and details are displayed in a
# small web page located at:
#
#    http(s)://[router IP address]:port/ext/stm-monitor.htm
#
# The script assumes that the time on your router is accurate to within
# a minute or so, so make sure your NTP settings are correct! It also only
# works correcly if your router is left on throughout each STM period,
# because rebooting destroys the Tomato bandwidth stats this script relies on.

# Logging command
LOGGER="logger -t STM-Monitor"

# STM Monitor version
stm_monitor_version=`head /tmp/stm-monitor.sh | grep "STM Monitor" | sed 's/# //g'`

# Check command line arguments

if [[ "$1" != "S5" && "$1" != "M10" && "$1" != "L10" && "$1" != "XL20" && "$1" != "ML20" && "$1" != "L30" && "$1" != "XL30" && "$1" != "XXL50" && "$1" != "XL60" && "$1" != "XXL100" && "$1" != "100" ]]
then
	$LOGGER "Usage: stm-monitor.sh [S5|M10|L10|XL20|ML20|L30|XL30|XXL50|XL60|XXL100|100]"
	exit
fi



#How much should QoS decrease the bandwidth to in % (good values are anything from 75 to 90)
qos_in_percentage=86
qos_out_percentage=84

# Override settings with user-defined values if valid values were specified
if [[ -e /jffs/stm-monitor.cfg ]]
then
	source /jffs/stm-monitor.cfg
fi

if [[ -e /tmp/stm-monitor.cfg ]]
then
	source /tmp/stm-monitor.cfg
fi

if [[ -e /tmp/stm-monitor.var ]]
then
	source /tmp/stm-monitor.var
else
	echo   "period_finals_1=-
period_finals_1_percent=-
period_finals_2=-
period_finals_2_percent=-
period_finals_3=-
period_finals_3_percent=-"> /tmp/stm-monitor.var
		source /tmp/stm-monitor.var
	fi

if [[ "$STM_PERIOD_BW_LIMIT_1"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_PERIOD_BW_LIMIT_1`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_PERIOD_BW_LIMIT_1 (${STM_PERIOD_BW_LIMIT_1} MB). Reverting to default value (${stm_period_bw_limit_mb_1} MB)."
	else
		stm_period_bw_limit_mb_1=$STM_PERIOD_BW_LIMIT_1
	fi
fi

if [[ "$STM_PERIOD_BW_LIMIT_2"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_PERIOD_BW_LIMIT_2`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_PERIOD_BW_LIMIT_2 (${STM_PERIOD_BW_LIMIT_2} MB). Reverting to default value (${stm_period_bw_limit_mb_2} MB)."
	else
		stm_period_bw_limit_mb_2=$STM_PERIOD_BW_LIMIT_2
	fi
fi

if [[ "$STM_PERIOD_BW_LIMIT_3"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_PERIOD_BW_LIMIT_3`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_PERIOD_BW_LIMIT_3 (${STM_PERIOD_BW_LIMIT_3} MB). Reverting to default value (${stm_period_bw_limit_mb_3} MB)."
	else
		stm_period_bw_limit_mb_3=$STM_PERIOD_BW_LIMIT_3
	fi
fi


if [[ "$STM_INBOUND_BANDWIDTH_WITHOUT_STM"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_INBOUND_BANDWIDTH_WITHOUT_STM`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_INBOUND_BANDWIDTH_WITHOUT_STM (${STM_INBOUND_BANDWIDTH_WITHOUT_STM} kb/s). Reverting to default value (${inbound_bandwidth_without_stm} kb/s)."
	else
		inbound_bandwidth_without_stm=$STM_INBOUND_BANDWIDTH_WITHOUT_STM
	fi
fi

if [[ "$STM_INBOUND_BANDWIDTH_WITH_STM"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_INBOUND_BANDWIDTH_WITH_STM`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_INBOUND_BANDWIDTH_WITH_STM (${STM_INBOUND_BANDWIDTH_WITH_STM} kb/s). Reverting to default value (${inbound_bandwidth_with_stm} kb/s)."
	else
		inbound_bandwidth_with_stm=$STM_INBOUND_BANDWIDTH_WITH_STM
	fi
fi

if [[ "$STM_OUTBOUND_BANDWIDTH_WITHOUT_STM"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_OUTBOUND_BANDWIDTH_WITHOUT_STM`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_OUTBOUND_BANDWIDTH_WITHOUT_STM (${STM_OUTBOUND_BANDWIDTH_WITHOUT_STM} kb/s). Reverting to default value (${outbound_bandwidth_without_stm} kb/s)."
	else
		outbound_bandwidth_without_stm=$STM_OUTBOUND_BANDWIDTH_WITHOUT_STM
	fi
fi

if [[ "$STM_OUTBOUND_BANDWIDTH_WITH_STM"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value < 0) {print(1)} }' value=$STM_OUTBOUND_BANDWIDTH_WITH_STM`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid STM_OUTBOUND_BANDWIDTH_WITH_STM (${STM_OUTBOUND_BANDWIDTH_WITH_STM} kb/s). Reverting to default value (${outbound_bandwidth_with_stm} kb/s)."
	else
		outbound_bandwidth_with_stm=$STM_OUTBOUND_BANDWIDTH_WITH_STM
	fi
fi


if [[ "$QOS_IN_PERCENTAGE"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value > 100 || value < 0) {print(1)} }' value=$QOS_IN_PERCENTAGE`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid QOS_IN_PERCENTAGE (${QOS_IN_PERCENTAGE}%). Reverting to default value (${qos_in_percentage}%)."
	else
		qos_in_percentage=$QOS_IN_PERCENTAGE
	fi
fi

if [[ "$QOS_OUT_PERCENTAGE"  != "" ]]
then
	invalid_value=`echo "" | awk '{ if (value > 100 || value < 0) {print(1)} }' value=$QOS_OUT_PERCENTAGE`
	if [[  "$invalid_value" = 1 ]]
	then
		$LOGGER "Invalid QOS_OUT_PERCENTAGE (${QOS_OUT_PERCENTAGE}%). Reverting to default value (${qos_out_percentage}%)."
	else
		qos_out_percentage=$QOS_OUT_PERCENTAGE
	fi
fi


# The upload QOS bandwidth values are set at ~91% of the cable modem
# bandwidth limiter (download set at 100%), which works well for my
# connection. Cable connections tend to be pretty stable so they should work
# ok for most users. The STM related variables are based on the information
# published by VM here: http://allyours.virginmedia.com/html/internet/traffic.html

# STM period definitions (time values are hours of the day e.g. 16 = 16:00 = 4pm)
stm_period_name_1="Daytime Down"
stm_period_start_1=10
stm_period_end_1=15
stm_period_direction_1="RX"

stm_period_name_2="Evening Down"
stm_period_start_2=17
stm_period_end_2=22
stm_period_direction_2="RX"

stm_period_name_3="Evening Up"
stm_period_start_3=15
stm_period_end_3=20
stm_period_direction_3="TX"

# STM sentence length (i.e. how long STM lasts for once it's triggered)
stm_sentence_length_hours=5
stm_sentence_length_seconds=`expr $stm_sentence_length_hours \* 3600`

# Definition of P2P/Newsgroups limitations
p2p_period_week_starts=17
p2p_period_week_ends=23 #at 00 it switched off
p2p_period_weekend_starts=12
p2p_period_weekend_ends=23 #at 00 it's switched off
#
fixed_limit_desc="Weedays: 5pm to 12am <BR> Weekends 12pm to 12am"

# Settings specific to 120 (120Mb down, 12Mb up)
if [[ $1 = "120" ]]
then
	downs=122880
	ups=12288
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=22000
	stm_period_bw_limit_mb_2=11000
	stm_period_bw_limit_mb_3=14000
	
# Settings specific to 100 (100Mb down, 10Mb up)
elif [[ $1 = "100" ]]
then
	downs=102400
	ups=10240
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=20000
	stm_period_bw_limit_mb_2=10000
	stm_period_bw_limit_mb_3=12000

# Settings specific to XXL100 (100Mb down, 5Mb up)

elif [[ $1 = "XXL100" ]]
then
	downs=102400
	ups=5120
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 65`
	# Speeds in MByte
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=20000
	stm_period_bw_limit_mb_2=10000
	stm_period_bw_limit_mb_3=6000

# Settings specific to XL60 (60Mb down, 6Mb up)
elif [[ $1 = "XL60" ]] 
then
	downs=61440
	ups=3072
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=10000
	stm_period_bw_limit_mb_2=5000
	stm_period_bw_limit_mb_3=4200

# Settings specific to XXL50 (50Mb down, 5Mb up)
elif [[ $1 = "XXL50" ]]
then
	downs=51200
	ups=5120
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 65`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=10000
	stm_period_bw_limit_mb_2=5000
	stm_period_bw_limit_mb_3=6000

# Settings specific to XL30 (30Mb down, 3Mb up)
elif [[ $1 = "XL30" ]] 
then
	downs=30720
	ups= 3072
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=7000
	stm_period_bw_limit_mb_2=3500
	stm_period_bw_limit_mb_3=4200

# Settings specific to L30 (30Mb down, 2Mb up)
elif [[ $1 = "L30" ]]
then
	downs=30720
	ups=2048
	reduction_in=`expr 100 - 40`
	reduction_out=`expr 100 - 75`
	# STM limits (values in Kbits)
	stm_period_bw_limit_mb_1=7000
	stm_period_bw_limit_mb_2=3500
	stm_period_bw_limit_mb_3=3000

	
########################################################################################


# Settings specific to XL20 (20Mb down, 2Mb up)
elif [[ $1 = "XL20" ]]
then
	downs=20480
	ups=2048
	reduction_in=`expr 100 - 75`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=7000
	stm_period_bw_limit_mb_2=3500
	stm_period_bw_limit_mb_3=3000
	
# Settings specific to ML20 (20Mb down, 1Mb up)
elif [[ $1 = "ML20" ]]
then
	downs=20480
	ups=1024
	reduction_in=`expr 100 - 75`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=7000
	stm_period_bw_limit_mb_2=3500
	stm_period_bw_limit_mb_3=1750

# Settings specific to L10 (10Mb down, 1Mb up)
elif [[ $1 = "L10" ]]
then
	downs=10240
	ups=1024
	reduction_in=`expr 100 - 75`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=3000
	stm_period_bw_limit_mb_2=1500
	stm_period_bw_limit_mb_3=1500

# Settings specific to M10 (10Mb down, 1Mb up)
elif [[ $1 = "M10" ]]
then
	downs=10240
	ups=1024
	reduction_in=`expr 100 - 75`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=1500
	stm_period_bw_limit_mb_2=750
	stm_period_bw_limit_mb_3=750
# Settings specific to S5 (5Mb down, 512Kb up)
elif [[ $1 = "S5" ]]
then
	downs=5120
	ups=256
	reduction_in=`expr 100 - 75`
	reduction_out=`expr 100 - 75`
	# STM limits (values in MB)
	stm_period_bw_limit_mb_1=500
	stm_period_bw_limit_mb_2=250
	stm_period_bw_limit_mb_3=100
fi
	# QOS settings (values in Kbits/s)
	inbound_bandwidth_with_stm=`expr $downs \* $reduction_in / 100 \* $qos_in_percentage / 100`
	outbound_bandwidth_with_stm=`expr $ups \* $reduction_out / 100 \* $qos_out_percentage / 100`
	inbound_bandwidth_without_stm=`expr $downs \* $qos_in_percentage / 100`
	outbound_bandwidth_without_stm=`expr $ups \* $qos_out_percentage / 100`
	


# Grab the current time
current_hours=`date +%H`
current_minutes=`date +%M`
current_unix_time=`date +%s`
current_week_day=`date +%u`
today=`date +%A`


if [[ $current_hours -ge 1 && $current_hours -lt $stm_period_start_1 ]]
then
		echo   "period_finals_1=-
period_finals_1_percent=-
period_finals_2=-
period_finals_2_percent=-
period_finals_3=-
period_finals_3_percent=-"> /tmp/stm-monitor.var
fi

#web_page=" "

# Set up web page. Use a string to store web page content and pipe
# it to /var/wwwext/stm-monitor.htm when the script exits
if [[ ! -e "/var/wwwext" ]]
then
	mkdir /var/wwwext
fi
web_page="${web_page}<html><head><title>`nvram get wan_hostname`.`nvram get wan_domain` - ${stm_monitor_version} - Status at ${current_hours}:${current_minutes} ${today}</title><meta http-equiv=\"refresh\" content=\"120\"</head>"

web_page="${web_page}<style type=\"text/css\">
<!--
body {background-color: #fff; color: #333; font-family: Verdana, Arial, Helvetica, sans-serif;
font-size: 10px; line-height: 1.0;}
h2 {color: #333; font-size: 16px;}
h3 {color: #333; font-size: 14px;}
h4 {color: #333; font-size: 12px;}
a {color: #9966cc; font-weight: bold; text-decoration: none;}
a:hover {color: #336699; font-weight: bold; text-decoration: none;}
img {border: 0px solid #cccccc; }
#progressbars {background: #fff; font-family: verdana,arial,sans-serif; font-size: 10px;
color: #000;}
.title {font-weight: bold; font-size: 10px;}

pre {
font-family: courier, Lucida Console, 
white-space: pre-wrap;
monospace;
font-size:9pt;
color:#555555; 
}

.prog-border {
height: 18px;
width: 120px;
background: #eeeee;
border: 1px solid #555577;
padding: 0px;
font-family: verdana,arial,sans-serif; font-size: 10px; color: #000;}

.prog-border2 {
height: 18px;
width: 120px;
background: #eeeee;
border: 1px solid #D20000;
padding: 0px;
font-family: verdana,arial,sans-serif; font-size: 10px; color: #000;}

.prog-border3 {
height: 18px;
width: 120px;
background: #eeeee;
border: 1px solid #DDDDDD;
padding: 0px;
font-family: verdana,arial,sans-serif; font-size: 10px; color: #000;}


.prog-bar {
height: 18px;
width: 118px
padding: 0;
background: #BBBBCC;
font-family: verdana,arial,sans-serif; font-size: 14px; color: #555577;}

.prog-bar2 {
height: 18px;
width: 118px
padding: 0;
background: #FF6633;
font-family: verdana,arial,sans-serif; font-size: 14px; color: #333333}

.prog-bar3 {
height: 18px;
width: 118px
padding: 0;
background: #DDDDDD;
font-family: verdana,arial,sans-serif; font-size: 14px; color: #B0B0B0}

</style>"





# Check whether we are still serving an STM sentence that has completed
# Upstream
if [[ "`nvram get tx_stm_sentence_active`" = 1  && $current_unix_time -gt "`nvram get tx_stm_sentence_end_unix_time`" ]]
then
	# Return QOS configuration to non-STM settings
	$LOGGER "STM (TX) sentence served: applying normal outbound QOS configuration"
	nvram set qos_obw=$outbound_bandwidth_without_stm
	service qos restart

	# Record the time at which this sentence ended
	nvram set tx_last_stm_sentence_end_unix_time=`nvram get tx_stm_sentence_end_unix_time`
	nvram set tx_last_stm_sentence_end_time=`nvram get tx_stm_sentence_end_time`

	# Unset our NVRAM variables
	nvram unset tx_stm_sentence_active
	nvram unset tx_stm_sentence_end_time
	nvram unset tx_stm_sentence_end_unix_time        
fi
# Downstream
if [[ "`nvram get rx_stm_sentence_active`" = 1  && $current_unix_time -gt "`nvram get rx_stm_sentence_end_unix_time`" ]]
then
	# Return QOS configuration to non-STM settings
	$LOGGER "STM (RX) sentence served: applying normal inbound QOS configuration"
	nvram set qos_ibw=$inbound_bandwidth_without_stm
	service qos restart

	# Record the time at which this sentence ended
	nvram set rx_last_stm_sentence_end_unix_time=`nvram get rx_stm_sentence_end_unix_time`
	nvram set rx_last_stm_sentence_end_time=`nvram get rx_stm_sentence_end_time`

	# Unset our NVRAM variables
	nvram unset rx_stm_sentence_active
	nvram unset rx_stm_sentence_end_time
	nvram unset rx_stm_sentence_end_unix_time        
fi

# Grab current statistics from rstats and wait for the file to be written
stats_file=/var/spool/rstats-speed.js
killall -USR1 rstats
sleep 2

# Read the stats file and extract the parts we're interested in (WAN RX and TX)
wan_interface=`nvram get wan_ifname` # Usually vlan1, but not always
read_next_rx_tx=0
while read line
do
	case $line in
		*$wan_interface*) read_next_rx_tx=1 ;;
	esac

	if [[ $read_next_rx_tx = 1 ]]
	then
		case $line in
			*rx:*) rx_data=$line ;;
			*tx:*) tx_data=$line ; read_next_rx_tx=0 ;;
		esac
	fi
done < $stats_file
rm $stats_file

# Check which STM periods are active, check total TX/RX against
# STM limits and note if an STM limit was exceeded
tx_stm_sentence_triggered=0
rx_stm_sentence_triggered=0
add_adjustment_note=0
web_page="${web_page}<table cellpadding=6 cellspacing=3 border=0 width=1000>"
web_page="${web_page}<tr><th colspan=7><font style=\"color:555577\" size=2>---=={ `nvram get wan_hostname`.`nvram get wan_domain` }==---</font></th></tr><tr align=\"center\" valign=\"middle\" bgcolor=\"555577\"><th colspan=4 width=45%><b style=\"color:white\">${stm_monitor_version}<b></th>"

web_page="${web_page}<th colspan=3><b style=\"color:white\">Status at ${current_hours}:${current_minutes} - ${today}<b></th></tr>"


web_page="${web_page}<tr align=\"center\" bgcolor=\"DDDDFF\"><th>STM Description<th>Start</th><th>End</th><th width=90>Quota MB</th><th width=90>Used MB</th><th>Average Rate</th><th width=120>Usage %</th></tr>"
for i in 1 2 3; do
	stm_period_name=`eval echo "\\$stm_period_name_$i"`
	stm_period_start=`eval echo "\\$stm_period_start_$i"`
	stm_period_end=`eval echo "\\$stm_period_end_$i"`
	stm_period_direction=`eval echo "\\$stm_period_direction_$i"`
	stm_period_bw_limit_mb=`eval echo "\\$stm_period_bw_limit_mb_$i"`
	
	if [[ $current_hours -ge $stm_period_start && $current_hours -lt $stm_period_end ]]
	then
		# This STM period is active.	
		web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\">"
		web_page="${web_page}<td align=\"left\">${stm_period_name} (${1})</td><td>${stm_period_start}:00</td><td>${stm_period_end}:00</td><td>${stm_period_bw_limit_mb}</td>"

		# Setup a few RX/TX dependent variables
		if [[ $stm_period_direction = "RX" ]]
		then
			data=$rx_data
			total_data=$rx_data
			stm_period_rx_active=1
			stm_sentence_active=`nvram get rx_stm_sentence_active`
			last_stm_sentence_end_unix_time=`nvram get rx_last_stm_sentence_end_unix_time`
		else
			data=$tx_data
			total_data=$tx_data
			stm_period_tx_active=1
			stm_sentence_active=`nvram get tx_stm_sentence_active`
			last_stm_sentence_end_unix_time=`nvram get tx_last_stm_sentence_end_unix_time`
		fi

		# Calculate the number of minutes since the STM period started, or since the last STM sentence
		# ended (whichever happened more recently).
		# Then calculate the number of values to take from the rstats data (each value covers two minutes)
		minutes_since_stm_period_started=`echo "" | awk '{ printf("%0.f", (((hours - stm_start) * 60) + minutes)); }' hours=$current_hours minutes=$current_minutes stm_start=${stm_period_start}`
		if [[ "$last_stm_sentence_end_unix_time" != "" ]]
		then
			minutes_since_last_stm_sentence_ended=`echo "" | awk '{ printf("%0.f", (current - sentence_end) / 60); }' current=$current_unix_time sentence_end=$last_stm_sentence_end_unix_time`
			if [[ $minutes_since_last_stm_sentence_ended -lt $minutes_since_stm_period_started ]]
			then
				# Need to calculate the number of seconds to subtract from the length of the
				# STM period when we do the maximum average rate calculation later on
				max_rate_adjustment=`echo "" | awk '{ print((mins_since_stm_start - mins_since_sentence_end) * 60); }' mins_since_stm_start=$minutes_since_stm_period_started mins_since_sentence_end=$minutes_since_last_stm_sentence_ended`
				number_of_values=`expr $minutes_since_last_stm_sentence_ended / 2`
			else
				# Don't need to adjust the rate calculation
				max_rate_adjustment=0
				number_of_values=`expr $minutes_since_stm_period_started / 2`	
			fi 
		else
			# Don't need to adjust the rate calculation
			max_rate_adjustment=0
			number_of_values=`expr $minutes_since_stm_period_started / 2`
		fi
		
		# Extract the useful numbers from the rstats data
		if [[ $number_of_values != 0 ]]
		then
			# Sum the last XXX values of the 720 in the rstats data to get the
			# total bytes transferred since the STM period started (or since the last
			# STM sentence ended) 
			total_transferred_bytes=`echo $data | awk '
				{
					values_string = substr($0, index($0, "[") + 1, index($0, "]") - 6);
					split(values_string, values_array, ",");
					for (i=(721-number_of_values); i<=720; i++) {
						total += values_array[i];
					} printf("%0.f" total);
				}' number_of_values=$number_of_values total=0`
			total_transferred_mb=`echo "" | awk '{ printf("%.0f", bytes / 1048576) }' bytes=$total_transferred_bytes`	
			average_rate=`echo "" | awk '{ printf("%.0f", bytes / (number_of_values * 2 * 60 * 1024)) }' number_of_values=$number_of_values bytes=$total_transferred_bytes`
			average_rate_kilobits=`echo "" | awk '{printf("%.0f", rate_kilobytes * 8)}' rate_kilobytes=$average_rate`
		else
			total_transferred_bytes="0"
			total_transferred_mb="0"
			average_rate="0"
			average_rate_kilobits="0"
		fi	
	
		# Calculate maximum average rate (i.e. rate required to trigger STM),
		# then compare this to our actual rate. If the actual rate is equal or
		# higher, we paint the cell red, if it's greater than 90% of the max
		# we paint it amber, and if it's anything else we paint it green.
		stm_period_max_ave_rate=`echo "" | awk '{printf((limit * 1024) / (((end - start)*3600) - adjustment))}' limit=$stm_period_bw_limit_mb start=$stm_period_start end=$stm_period_end adjustment=$max_rate_adjustment`
		cell_color=`echo | awk '{ if (rate >= max) {print("FF6633")} else if (rate > (0.9 * max)) {print("FFE47A")} else {print("AAFF88")}}' rate=$average_rate max=$stm_period_max_ave_rate`
		
		# Add data to web page
		total_transferred_mb_percent=`echo "" | awk '{ printf("%.0f", (transferred / limit) * 100) }' transferred=$total_transferred_mb limit=$stm_period_bw_limit_mb`
		under="<div class=\"prog-border\"><div class=\"prog-bar\" style=\"width:$total_transferred_mb_percent%\">$total_transferred_mb_percent%</div></div></td>"
		over="<div class=\"prog-border2\"><div class=\"prog-bar2\" style=\"width:100%\">$total_transferred_mb_percent%</div></div></td>"
		if [[ $max_rate_adjustment != 0 ]]
		then
		    # Add link to note explaining adjusted figures
			web_page="${web_page}<td>${total_transferred_mb} *</td><td bgcolor=\"${cell_color}\">${average_rate} KB/s (${average_rate_kilobits} kb/s) *</td><td align=left>"
			if [[ $total_transferred_mb_percent -le 99 ]] 
			then 
			web_page="${web_page}$under"
			else
			web_page="${web_page}$over"
			fi
		    add_adjustment_note=1
		else
		    web_page="${web_page}<td>${total_transferred_mb} </td><td bgcolor=\"${cell_color}\">${average_rate} KB/s (${average_rate_kilobits} kb/s)</td><td align=left>"
			if [[ $total_transferred_mb_percent -le 99 ]] 
			then 
			web_page="${web_page}$under"
			else
			web_page="${web_page}$over"
			fi
			fi
		
		# Check whether STM limit has been exceeded
		stm_period_bw_limit_bytes=`expr $stm_period_bw_limit_mb \* 1048576`
		result=`echo "" | awk '{ if (total > limit) {print(1)} else {print(0)}}' total=$total_transferred_bytes limit=$stm_period_bw_limit_bytes`
		if [[ $result = 1 && "$stm_sentence_active" != 1 ]]
		then
			if [[ $stm_period_direction = "RX" ]]
			then
				rx_stm_sentence_triggered=1
				nvram set rx_stm_sentence_active=1
			else
				tx_stm_sentence_triggered=1
				nvram set tx_stm_sentence_active=1
			fi
			$LOGGER "STM (${stm_period_direction}) triggered: ${stm_period_name} (${stm_period_start}:00 -> ${stm_period_end}:00, ${stm_period_bw_limit_mb} MB)"
		fi

		if [[ $i = 1 ]]
		then
				sed -i "s/^period_finals_1=.*/period_finals_1=$total_transferred_mb/g" /tmp/stm-monitor.var
				sed -i "s/^period_finals_1_percent=.*/period_finals_1_percent=$total_transferred_mb_percent/g" /tmp/stm-monitor.var		
		elif [[ $i = 2 ]]
		then
				sed -i "s/^period_finals_2=.*/period_finals_2=$total_transferred_mb/g" /tmp/stm-monitor.var
				sed -i "s/^period_finals_2_percent=.*/period_finals_2_percent=$total_transferred_mb_percent/g" /tmp/stm-monitor.var
		elif [[ $i = 3 ]]
		then
				sed -i "s/^period_finals_3=.*/period_finals_3=$total_transferred_mb/g" /tmp/stm-monitor.var
				sed -i "s/^period_finals_3_percent=.*/period_finals_3_percent=$total_transferred_mb_percent/g" /tmp/stm-monitor.var		
		fi

		
	else
		# This STM period is not currently active
		web_page="${web_page}<tr bgcolor=\"EEEEEE\" style=\"color:BBBBBB\" align=\"center\">"
		if [[ $i = 1 ]]
		then
				if [[ $period_finals_1_percent -gt 100 ]]
				then 
				extra1="*"
				greybar="<td align=left><div class=\"prog-border3\"><div class=\"prog-bar3\" style=\"width:100%\">$period_finals_1_percent%</div></div></td>"
				elif [[ $period_finals_1_percent = - ]]
				then
				greybar="<td>-</td>"
				else
				greybar="<td align=left><div class=\"prog-border3\"><div class=\"prog-bar3\" style=\"width:$period_finals_1_percent%\">$period_finals_1_percent%</div></div></td>"
			fi
		web_page="${web_page}<td align=\"left\">${stm_period_name} (${1})</td><td>${stm_period_start}:00</td><td>${stm_period_end}:00</td><td>${stm_period_bw_limit_mb}</td><td>$period_finals_1 $extra1</td><td>Period inactive</td>$greybar</td>"
		
		elif [[ $i = 2 ]]
		then
				if [[ $period_finals_2_percent -gt 100 ]]
				then 
				extra2="*"
				greybar="<td align=left><div class=\"prog-border3\"><div class=\"prog-bar3\" style=\"width:100%\">$period_finals_2_percent%</div></div></td>"
				elif [[ $period_finals_2_percent = - ]]
				then
				greybar="<td>-</td>"
				else
				greybar="<td align=left><div class=\"prog-border3\"><div class=\"prog-bar3\" style=\"width:$period_finals_2_percent%\">$period_finals_2_percent%</div></div></td>"
			fi
		web_page="${web_page}<td align=\"left\">${stm_period_name} (${1})</td><td>${stm_period_start}:00</td><td>${stm_period_end}:00</td><td>${stm_period_bw_limit_mb}</td><td>$period_finals_2 $extra2</td><td>Period inactive</td>$greybar</td>"	

		elif [[ $i = 3 ]]
		then
				if [[ $period_finals_3_percent -gt 100 ]]
				then 
				extr3a="*"
				greybar="<td align=left><div class=\"prog-border3\"><div class=\"prog-bar3\" style=\"width:100%\">$period_finals_3_percent%</div></div></td>"
				elif [[ $period_finals_3_percent = - ]]
				then
				greybar="<td>-</td>"
				else
				greybar="<td align=left><div class=\"prog-border3\"><div class=\"prog-bar3\" style=\"width:$period_finals_3_percent%\">$period_finals_3_percent%</div></div></td>"
			fi
		web_page="${web_page}<td align=\"left\">${stm_period_name} (${1})</td><td>${stm_period_start}:00</td><td>${stm_period_end}:00</td><td>${stm_period_bw_limit_mb}</td><td>$period_finals_3 $extra3</td><td>Period inactive</td>$greybar</td>"	
		
		fi
	fi
	web_page="${web_page}</tr>"
done


# If some of the rate figures were adjusted due to a recent STM setence, add a note to make this clear
if [[ $add_adjustment_note = 1 ]]
then
    web_page="${web_page}<tr><th bgcolor=\"EEEEEE\" colspan=7 align=left><small>*This data was calculated from the end of the last STM sentence (`nvram get rx_last_stm_sentence_end_time`), rather than from the beginning of the STM period (${stm_period_start_2}:00).</small</th></tr>"
fi

web_page="${web_page}</table>"

# If STM is active, check whether QOS is set accordingly
# Upstream
if [[ "`nvram get tx_stm_sentence_active`" = 1  && "`nvram get qos_obw`" != $outbound_bandwidth_with_stm ]]
then
	if [[ $tx_stm_sentence_triggered = 1 ]]
	then
		# We must have triggered an STM sentence on this run.
		# Work out the hour component of the time the STM sentence will end
		# (for logging purposes only - we will use unix time to track the
		# STM sentence accurately)
		tx_stm_end_hour=`expr $current_hours + $stm_sentence_length_hours`
		if [[ $tx_stm_end_hour -gt 23 ]]
		then
			tx_stm_end_hour="0`expr $tx_stm_end_hour - 24`"
		fi
		
		# Set NVRAM variables to describe the STM sentence being served
		nvram set tx_stm_sentence_end_time="$tx_stm_end_hour:$current_minutes"
		nvram set tx_stm_sentence_end_unix_time=`expr $current_unix_time + $stm_sentence_length_seconds`
		$LOGGER "STM (TX) sentence started: applying STM outbound QOS configuration (until $tx_stm_end_hour:$current_minutes)"
	else
		# An STM sentence is active and we didn't trigger it on this run,
		# but the QOS settings are wrong. We must have rebooted, or someone
		# messed with the QOS settings. So we want to set the QOS settings
		# to deal with the STM, but we don't want to overwrite our NVRAM
		# variables.
		$LOGGER "STM (TX) sentence resumed: applying STM outbound QOS configuration (until `nvram get tx_stm_sentence_end_time`)" 
	fi
	
	# Apply STM QOS settings
	nvram set qos_obw=$outbound_bandwidth_with_stm
	service qos restart
fi
# Downstream
if [[ "`nvram get rx_stm_sentence_active`" = 1  && "`nvram get qos_ibw`" != $inbound_bandwidth_with_stm ]]
then
	if [[ $rx_stm_sentence_triggered = 1 ]]
	then
		# We must have triggered an STM sentence on this run.
		# Work out the hour component of the time the STM sentence will end
		# (for logging purposes only - we will use unix time to track the
		# STM sentence accurately)
		rx_stm_end_hour=`expr $current_hours + $stm_sentence_length_hours`
		if [[ $rx_stm_end_hour -gt 23 ]]
		then
			rx_stm_end_hour="0`expr $rx_stm_end_hour - 24`"
		fi
		
		# Set NVRAM variables to describe the STM sentence being served
		nvram set rx_stm_sentence_end_time="$rx_stm_end_hour:$current_minutes"
		nvram set rx_stm_sentence_end_unix_time=`expr $current_unix_time + $stm_sentence_length_seconds`


		$LOGGER "STM (RX) sentence started: applying STM inbound QOS configuration (until $rx_stm_end_hour:$current_minutes)"
	else
		# An STM sentence is active and we didn't trigger it on this run,
		# but the QOS settings are wrong. We must have rebooted, or someone
		# messed with the QOS settings. So we want to set the QOS settings
		# to deal with the STM, but we don't want to overwrite our NVRAM
		# variables.
		$LOGGER "STM (RX) sentence resumed: applying STM inbound QOS configuration (until `nvram get rx_stm_sentence_end_time`)" 
	fi
	
	# Apply STM QOS settings
	nvram set qos_ibw=$inbound_bandwidth_with_stm
	service qos restart
fi

#web_page="${web_page}STM Mitigation:"
web_page="${web_page}<table cellpadding=6 cellspacing=3 border=0 width=1000>"
web_page="${web_page}<tr align=\"center\" bgcolor=\"DDDDFF\" style=\"color:000000\"><th width=22%><b>STM Mitigation</b><th width=39%>Download</th><th width=39%>Upload</th></tr>"

if [[ "`nvram get tx_stm_sentence_active`" = 1 ]]
then
	if [[ "`nvram get rx_stm_sentence_active`" = 1 ]]
	then
		# Serving an STM sentence for both RX and TX
		web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td><b style=\"color:FF6633\">Active</b></td><td><b style=\"color:FF6633\">STM sentence will end at: `nvram get rx_stm_sentence_end_time`</b></td><td><b style=\"color:FF6633\">STM sentence will end at `nvram get tx_stm_sentence_end_time`</b></td></tr>"
	else
		# Serving an STM sentence for TX only
		web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td><b style=\"color:FF6633\">Active</b></td><td><b style=\"color:BBBBBB\">Not affected</b></td><td><b style=\"color:FF6633\">STM sentence will end at `nvram get tx_stm_sentence_end_time`</b></td></tr>"
	fi
else
	if [[ "`nvram get rx_stm_sentence_active`" = 1 ]]
	then
		# Serving an STM sentence for RX only
		web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td><b style=\"color:FF6633\">Active</b></td><td><b style=\"color:FF6633\">STM sentence will end at `nvram get rx_stm_sentence_end_time`</b></td><td><b style=\"color:BBBBBB\">Not affected</b></td></tr>"
	else
		# Not serving an STM sentence
		web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td><b style=\"color:green\">Inactive</b></td><td><b style=\"color:BBBBBB\">Not affected</b></td><td><b style=\"color:BBBBBB\">Not affected</b></td></tr>"
	fi
fi

web_page="${web_page}<table cellpadding=6 cellspacing=3 border=0 width=1000>"
web_page="${web_page}<tr align=\"center\" bgcolor=\"DDDDFF\"><th><b>Contract $1</b><th>Download kb/s</th><th>Upload kb/s</th><th width=120>%</th><th>Description</th></tr>"
web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td align=\"left\">Physical limits</td><td>${downs}</td><td>${ups}</td><td>100% / 100%</td><td align=left>Theoretical throughput</td></tr>"
web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\"><td align=\"left\">Current QoS values</td><td>"

if [ `nvram get qos_ibw` == ${inbound_bandwidth_with_stm} ] 
then
left=`nvram get rx_stm_sentence_end_unix_time`
left=`expr $left - $current_unix_time`
left=`date -d @$left +%H:%M`
web_page="${web_page}<b style=\"color:FF6633\">`nvram get qos_ibw`</b><br>(ends in $left hours)"
else
web_page="${web_page}<b style=\"color:blue\">`nvram get qos_ibw`</b>"
fi
web_page="${web_page}</td><td>"

if [ `nvram get qos_obw` == ${outbound_bandwidth_with_stm} ]
then
left=`nvram get tx_stm_sentence_end_unix_time`
left=`expr $left - $current_unix_time`
left=`date -d @$left +%H:%M`
web_page="${web_page}<b style=\"color:FF6633\">`nvram get qos_obw`</b><br>(ends in $left hours)"
else
web_page="${web_page}<b style=\"color:blue\">`nvram get qos_obw`</b>"
fi
web_page="${web_page}</td align=left><td align=left>"


if [ `nvram get qos_ibw` == ${inbound_bandwidth_with_stm} ] 
then
web_page="${web_page}<div class=\"prog-border2\"><div class=\"prog-bar2\" style=\"width:`expr $inbound_bandwidth_with_stm \* 100 / $downs`%\">`expr $inbound_bandwidth_with_stm \* 100 / $downs`%</div></div>"
else
temp=`nvram get qos_ibw`
web_page="${web_page}<div class=\"prog-border\"><div class=\"prog-bar\" style=\"width:`expr $temp \* 100 / $downs`%\">`expr $temp \* 100 / $downs`%</div></div>"
fi

if [ `nvram get qos_obw` == ${outbound_bandwidth_with_stm} ]
then
web_page="${web_page}<div class=\"prog-border2\"><div class=\"prog-bar2\" style=\"width:`expr $outbound_bandwidth_with_stm \* 100 / $ups`%\">`expr $outbound_bandwidth_with_stm \* 100 / $ups`%</div></div>"
else
temp=`nvram get qos_obw`
web_page="${web_page}<div class=\"prog-border\"><div class=\"prog-bar\" style=\"width:`expr $temp \* 100 / $ups`%\">`expr $temp \* 100 / $ups`%</div></div>"
fi

web_page="${web_page}</td><td align=left>Router actuals</td></tr>"

web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td align=\"left\">STM limits </td><td>`expr $downs \* $reduction_in / 100`</td><td>`expr $ups \* $reduction_out / 100`</td><td>$reduction_in% / $reduction_out%</td><td align=left>Imposed during STM</td></tr>"
# Add latest log entries to web page
web_page="${web_page}<tr align=\"center\" bgcolor=\"DDDDFF\"><th colspan=4>Application based restriction (throughput independent)</th><TD>Active periods</TD></tr>"

#<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td align=\"left\">
web_page="${web_page}<tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td align=\"left\">P2P</td><th colspan=3 rowspan=2>"

p2p_period_week_starts=17
p2p_period_week_ends=23 #at 00 it switched off
p2p_period_weekend_starts=12
p2p_period_weekend_ends=23 #at 00 it's switched off

if [[ "$current_week_day" -ge 1 && "$current_week_day" -le 5 && "$current_hours" -ge "$p2p_period_week_starts" && "$current_hours" -le "$p2p_period_week_ends" ]] || [[ "$current_week_day"  -ge 6 && "$current_week_day" -le 7 && "$current_hours" -ge "$p2p_period_weekend_starts" && "$current_hours" -le "$p2p_period_weekend_ends" ]]
then
web_page="${web_page}<b style=\"color:blue\">ACTIVE</b><BR>Capped to `expr $downs \* $reduction_in / 100`/`expr $ups \* $reduction_out / 100` down/up</b>"
else
web_page="${web_page}INACTIVE<BR>"
fi
web_page="${web_page}</td><th align=left rowspan=2>$fixed_limit_desc</td></tr><tr bgcolor=\"EEEEEE\" align=\"center\" style=\"color:BBBBBB\"><td align=\"left\">Newsgroups</td>"
web_page="${web_page}</tr>"

web_page="${web_page}<tr align=\"center\" bgcolor=\"DDDDFF\"><th colspan=5>Latest log entries</th></tr>"
web_page="${web_page}<tr align=\"center\" bgcolor=\"EEEEEE\"><th colspan=5 align=\"left\"><pre id=\"log\">"
log="<nobr><tt>`cat \`ls -r /var/log/ | sed 's/messages/\/var\/log\/messages/'\` | grep STM-Monitor | sed 's/\$/<br>/g'`</tt></nobr>"
web_page="${web_page}${log}"
web_page="${web_page}</pre></th></tr><tr bgcolor=\"555577\"><th colspan=5><font style=\"color:white\" size=1>For additional details visit Tievolu's <A HREF=http://www.tievolu.co.uk/stm/stm-monitor.html style=\"color:white\"><u>STM homepage</u></a>, the tomato <A href=\"http://www.linksysinfo.org/index.php?threads/virgin-media-uk-stm-monitoring-script.28237\" style=\"color:white\"><u>forum</u></a> and VirginMedia fair-usage policies <a href=http://www.virginmedia.com/images/STM20MblargeA.jpg style=\"color:white\"><u>5-20</u></a>/<a href=http://www.virginmedia.com/images/STM30MblargeA.jpg style=\"color:white\">><u>30-120</u></a></font></th></tr></table><br>"

# Publish web page
web_page="${web_page}</body></html>"
echo $web_page > /var/wwwext/stm-monitor.htm
